/* mouseclick.c --- simulate a mouse click.

   Copyright (C) 2005 Ralph Schleicher

   This file is part of MTk, the Matlab widget toolkit.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

      * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

      * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in
        the documentation and/or other materials provided with the
        distribution.

      * Neither the name of the copyright holder nor the names of its
        contributors may be used to endorse or promote products derived
        from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
   FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
   COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
   ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.  */

/* Commentary:

   On Unix, compile with

        mex mouseclick.c -lXtst -lXext -lX11

   On Windows (Windows 95, Windows NT 3.1, or newer), compile with

        mex mouseclick.c

   On Windows 98, Windows NT 4.0 Service Pack 3, or newer, you can
   compile with

        mex -DHAVE_SENDINPUT mouseclick.c  */


#undef USE_X11
#undef USE_W32

#ifdef _WIN32
#define USE_W32 1
#else /* not _WIN32 */
#ifdef _WIN64
#define USE_W32 1
#else /* not _WIN64 */
#define USE_X11 1
#endif /* not _WIN64 */
#endif /* not _WIN32 */

#include <mex.h>

#if USE_X11
#include <X11/Xlib.h>
#include <X11/extensions/XTest.h>
#else /* not USE_X11 */
#if USE_W32
#include <windows.h>
#else /* not USE_W32 */
#error "Fix me"
#endif /* not USE_W32 */
#endif /* not USE_X11 */

void
mexFunction (int output_arg_count, mxArray *output_arg_vec[],
	     int input_arg_count, const mxArray *input_arg_vec[])
{
#if USE_X11

  Display *display;

  display = XOpenDisplay (NULL);
  if (display == NULL)
    mexErrMsgTxt ("Can not open display");

  XTestFakeButtonEvent (display, 1, True, 0);
  XTestFakeButtonEvent (display, 1, False, 1);

  XCloseDisplay (display);

#else /* not USE_X11 */
#if USE_W32
#if HAVE_SENDINPUT

  INPUT input;

  input.type = INPUT_MOUSE;
  input.mi.dx = 0;
  input.mi.dy = 0;
  input.mi.mouseData = 0;
  input.mi.time = 0;
  input.mi.dwExtraInfo = 0;

  input.mi.dwFlags = MOUSEEVENTF_LEFTDOWN;
  SendInput (1, &input, sizeof (INPUT));
  input.mi.dwFlags = MOUSEEVENTF_LEFTUP;
  SendInput (1, &input, sizeof (INPUT));

#else /* not HAVE_SENDINPUT */

  mouse_event (MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0);
  mouse_event (MOUSEEVENTF_LEFTUP, 0, 0, 0, 0);

#endif /* not HAVE_SENDINPUT */
#else /* not USE_W32 */

  (void) 0;

#endif /* not USE_W32 */
#endif /* not USE_X11 */
}
